#ifndef TEST_SCHEMA_HPP
#define TEST_SCHEMA_HPP

#if __cplusplus >= 201703L
#include <filesystem>
namespace fs = std::filesystem;
#elif __cplusplus == 201402L
#include <experimental/filesystem>
namespace fs = std::experimental::filesystem;
#else
#error Unsupported C++ standard
#endif

#include <file_options/file_options.hpp>
#include "keywords.hpp"
#include "defaults.hpp"
#include "parsers.hpp"

using file_options::converter;
using file_options::declare_file;
using file_options::declare_group;
using file_options::declare_list;
using file_options::declare_option;
using file_options::file_schema;
using file_options::opt_type;
using file_options::sec_type;

// define the configuration file structure and declare the supported options
const file_schema valid_options = declare_file({
        // section for global settings
        declare_section(
                keywords::global_settings, sec_type::mandatory,
                declare_group({

                        declare_option<bool>(
                                keywords::use_syslog, opt_type::mandatory,
                                converter<bool>(parsers::parse_bool)),

                        declare_option<fs::path>(
                                keywords::log_file, opt_type::optional,
                                converter<fs::path>(parsers::parse_path)),

                        declare_option<uint32_t>(
                                keywords::log_file_max_size, opt_type::optional,
                                converter<uint32_t>(parsers::parse_capacity)),

                        declare_option<std::string>(
                                keywords::transport_protocol,
                                opt_type::mandatory),

                        declare_option<std::string>(keywords::bind_address,
                                                    opt_type::mandatory),

                        declare_option<uint32_t>(
                                keywords::remote_port, opt_type::mandatory,
                                converter<uint32_t>(parsers::parse_number)),

                        declare_option<fs::path>(
                                keywords::pidfile, opt_type::mandatory,
                                converter<fs::path>(parsers::parse_path)),

                        declare_option<uint32_t>(
                                keywords::workers, opt_type::mandatory,
                                converter<uint32_t>(parsers::parse_number)),
                })),
});


#endif // TEST_SCHEMA_HPP
