#if __cplusplus >= 201703L
#include <filesystem>
namespace fs = std::filesystem;
#elif __cplusplus == 201402L
#include <experimental/filesystem>
namespace fs = std::experimental::filesystem;
#else
#error Unsupported C++ standard
#endif

#include "common/schema.hpp"

#include <catch2/catch.hpp>

TEST_CASE("Options are read from a configuration file ", "[config]") {

    fs::path filename{"test_config.yml"};

    file_options::options_map opt_map;
    file_options::parse_yaml_file(filename, valid_options, opt_map);

    // load global_settings
    const auto& global_settings = opt_map.get_as<file_options::options_group>(
            keywords::global_settings);

    auto use_syslog = global_settings.get_as<bool>(keywords::use_syslog);
    REQUIRE(use_syslog == false);

    fs::path log_file;
    uint32_t log_file_max_size;

    if(global_settings.has(keywords::log_file)) {
        log_file = global_settings.get_as<fs::path>(keywords::log_file);
        REQUIRE(log_file == "/usr/local/var/run/logfile.log");
    }

    if(global_settings.has(keywords::log_file_max_size)) {
        log_file_max_size =
                global_settings.get_as<uint32_t>(keywords::log_file_max_size);
    }

    auto transport_protocol =
            global_settings.get_as<std::string>(keywords::transport_protocol);
    REQUIRE(transport_protocol == "tcp");

    auto bind_address =
            global_settings.get_as<std::string>(keywords::bind_address);
    REQUIRE(bind_address == "127.0.0.1");

    auto remote_port = global_settings.get_as<uint32_t>(keywords::remote_port);
    REQUIRE(remote_port == 42000);

    auto daemon_pidfile = global_settings.get_as<fs::path>(keywords::pidfile);
    REQUIRE(daemon_pidfile == "/usr/local/var/run/pidfile.pid");

    auto workers_in_pool = global_settings.get_as<uint32_t>(keywords::workers);
    REQUIRE(workers_in_pool == 4);
}

